@@ -31,6 +31,7 @@
 #include <stdarg.h>
 #include <errno.h>
 #include <unistd.h>
+#include <fcntl.h>
 #include <stdint.h>
 #include <sys/stat.h>
 #include <dirent.h>
@@ -93,6 +94,7 @@ struct audio_adapter {
 	GIOChannel *hsp_ag_server;
 	GIOChannel *hfp_ag_server;
 	GIOChannel *hsp_hs_server;
+	GIOChannel *sco_server;
 };
 
 static DBusConnection *connection = NULL;
@@ -455,10 +457,6 @@ static void auth_cb(DBusError *derr, void *user_data)
 
 	if (derr && dbus_error_is_set(derr)) {
 		error("Access denied: %s", derr->message);
-		if (dbus_error_has_name(derr, DBUS_ERROR_NO_REPLY)) {
-			debug("Canceling authorization request");
-			btd_cancel_authorization(&device->src, &device->dst);
-		}
 
 		headset_set_state(device, HEADSET_STATE_DISCONNECTED);
 	} else {
@@ -472,6 +470,49 @@ static void auth_cb(DBusError *derr, void *user_data)
 	}
 }
 
+static void sco_server_cb(GIOChannel *chan, int err, const bdaddr_t *src,
+			const bdaddr_t *dst, gpointer data)
+{
+	int sk;
+	struct audio_device *device;
+	char addr[18];
+
+	if (err < 0) {
+		error("accept: %s (%d)", strerror(-err), -err);
+		return;
+	}
+
+	device = manager_find_device(dst, NULL, FALSE);
+	if (!device)
+		goto drop;
+
+	if (headset_get_state(device) < HEADSET_STATE_CONNECTED) {
+		debug("Refusing SCO from non-connected headset");
+		goto drop;
+	}
+
+	ba2str(dst, addr);
+
+	if (!get_hfp_active(device)) {
+		error("Refusing non-HFP SCO connect attempt from %s", addr);
+		goto drop;
+	}
+
+	sk = g_io_channel_unix_get_fd(chan);
+	fcntl(sk, F_SETFL, 0);
+
+	if (headset_connect_sco(device, chan) == 0) {
+		debug("Accepted SCO connection from %s", addr);
+		headset_set_state(device, HEADSET_STATE_PLAYING);
+	}
+
+	return;
+
+drop:
+	g_io_channel_close(chan);
+	g_io_channel_unref(chan);
+}
+
 static void ag_io_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 			const bdaddr_t *dst, gpointer data)
 {
@@ -524,7 +565,6 @@ static void ag_io_cb(GIOChannel *chan, int err, const bdaddr_t *src,
 drop:
 	g_io_channel_close(chan);
 	g_io_channel_unref(chan);
-	return;
 }
 
 static void hs_io_cb(GIOChannel *chan, int err, const bdaddr_t *src,
@@ -564,28 +604,33 @@ static int headset_server_init(struct audio_adapter *adapter)
 			hfp = tmp;
 	}
 
+	adapter->sco_server = bt_sco_listen(&adapter->src, 0, sco_server_cb,
+						adapter);
+	if (!adapter->sco_server) {
+		error("Unable to start SCO server socket");
+		return -1;
+	}
+
 	flags = RFCOMM_LM_AUTH | RFCOMM_LM_ENCRYPT;
 
 	if (master)
 		flags |= RFCOMM_LM_MASTER;
 
 	adapter->hsp_ag_server = bt_rfcomm_listen(&adapter->src, chan, flags,
-					ag_io_cb,adapter);
+							ag_io_cb, adapter);
 	if (!adapter->hsp_ag_server)
-		return -1;
+		goto failed;
 
 	record = hsp_ag_record(chan);
 	if (!record) {
 		error("Unable to allocate new service record");
-		return -1;
+		goto failed;
 	}
 
 	if (add_record_to_server(&adapter->src, record) < 0) {
 		error("Unable to register HS AG service record");
 		sdp_record_free(record);
-		g_io_channel_unref(adapter->hsp_ag_server);
-		adapter->hsp_ag_server = NULL;
-		return -1;
+		goto failed;
 	}
 	adapter->hsp_ag_record_id = record->handle;
 
@@ -597,26 +642,42 @@ static int headset_server_init(struct audio_adapter *adapter)
 	chan = DEFAULT_HF_AG_CHANNEL;
 
 	adapter->hfp_ag_server = bt_rfcomm_listen(&adapter->src, chan, flags,
-					ag_io_cb, adapter);
+							ag_io_cb, adapter);
 	if (!adapter->hfp_ag_server)
-		return -1;
+		goto failed;
 
 	record = hfp_ag_record(chan, features);
 	if (!record) {
 		error("Unable to allocate new service record");
-		return -1;
+		goto failed;
 	}
 
 	if (add_record_to_server(&adapter->src, record) < 0) {
 		error("Unable to register HF AG service record");
 		sdp_record_free(record);
-		g_io_channel_unref(adapter->hfp_ag_server);
-		adapter->hfp_ag_server = NULL;
-		return -1;
+		goto failed;
 	}
 	adapter->hfp_ag_record_id = record->handle;
 
 	return 0;
+
+failed:
+	if (adapter->sco_server) {
+		g_io_channel_unref(adapter->sco_server);
+		adapter->sco_server = NULL;
+	}
+
+	if (adapter->hsp_ag_server) {
+		g_io_channel_unref(adapter->hsp_ag_server);
+		adapter->hsp_ag_server = NULL;
+	}
+
+	if (adapter->hfp_ag_server) {
+		g_io_channel_unref(adapter->hfp_ag_server);
+		adapter->hfp_ag_server = NULL;
+	}
+
+	return -1;
 }
 
 static int gateway_server_init(struct audio_adapter *adapter)
@@ -673,15 +734,11 @@ static int audio_probe(struct btd_device *device, GSList *records)
 {
 	struct btd_adapter *adapter = device_get_adapter(device);
 	const gchar *path = device_get_path(device);
-	const char *source, *destination;
 	bdaddr_t src, dst;
 	struct audio_device *dev;
 
-	source = adapter_get_address(adapter);
-	destination = device_get_address(device);
-
-	str2ba(source, &src);
-	str2ba(destination, &dst);
+	adapter_get_address(adapter, &src);
+	device_get_address(device, &dst);
 
 	dev = manager_find_device(&dst, NULL, FALSE);
 	if (!dev) {
@@ -699,12 +756,11 @@ static int audio_probe(struct btd_device *device, GSList *records)
 static void audio_remove(struct btd_device *device)
 {
 	struct audio_device *dev;
-	const char *destination = device_get_address(device);
-	bdaddr_t dst;
+	bdaddr_t bdaddr;
 
-	str2ba(destination, &dst);
+	device_get_address(device, &bdaddr);
 
-	dev = manager_find_device(&dst, NULL, FALSE);
+	dev = manager_find_device(&bdaddr, NULL, FALSE);
 	if (!dev)
 		return;
 
@@ -728,11 +784,9 @@ static struct audio_adapter *get_audio_adapter(struct btd_adapter *adapter)
 {
 	struct audio_adapter *adp;
 	const gchar *path = adapter_get_path(adapter);
-	const char *source;
 	bdaddr_t src;
 
-	source = adapter_get_address(adapter);
-	str2ba(source, &src);
+	adapter_get_address(adapter, &src);
 
 	adp = find_adapter(adapters, path);
 	if (!adp) {
@@ -838,7 +892,21 @@ static int a2dp_server_probe(struct btd_adapter *adapter)
 	if (!adp)
 		return -EINVAL;
 
-	return a2dp_init(connection, config);
+	return a2dp_register(connection, &adp->src, config);
+}
+
+static void a2dp_server_remove(struct btd_adapter *adapter)
+{
+	struct audio_adapter *adp;
+	const gchar *path = adapter_get_path(adapter);
+
+	DBG("path %s", path);
+
+	adp = find_adapter(adapters, path);
+	if (!adp)
+		return;
+
+	return a2dp_unregister(&adp->src);
 }
 
 static int avrcp_server_probe(struct btd_adapter *adapter)
@@ -852,7 +920,21 @@ static int avrcp_server_probe(struct btd_adapter *adapter)
 	if (!adp)
 		return -EINVAL;
 
-	return avrcp_init(connection, config);
+	return avrcp_register(connection, &adp->src, config);
+}
+
+static void avrcp_server_remove(struct btd_adapter *adapter)
+{
+	struct audio_adapter *adp;
+	const gchar *path = adapter_get_path(adapter);
+
+	DBG("path %s", path);
+
+	adp = find_adapter(adapters, path);
+	if (!adp)
+		return;
+
+	return avrcp_unregister(&adp->src);
 }
 
 static struct btd_device_driver audio_driver = {
@@ -879,11 +961,13 @@ static struct btd_adapter_driver gateway_server_driver = {
 static struct btd_adapter_driver a2dp_server_driver = {
 	.name	= "audio-a2dp",
 	.probe	= a2dp_server_probe,
+	.remove	= a2dp_server_remove,
 };
 
 static struct btd_adapter_driver avrcp_server_driver = {
 	.name	= "audio-control",
 	.probe	= avrcp_server_probe,
+	.remove	= avrcp_server_remove,
 };
 
 int audio_manager_init(DBusConnection *conn, GKeyFile *conf)
@@ -999,9 +1083,6 @@ struct audio_device *manager_find_device(const bdaddr_t *bda, const char *interf
 {
 	GSList *l;
 
-	if (!bacmp(bda, BDADDR_ANY) && !interface && !connected)
-		return devices->data;
-
 	for (l = devices; l != NULL; l = l->next) {
 		struct audio_device *dev = l->data;
 
